package com.tcs.fusion.emp.eng.quartz;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.quartz.CronScheduleBuilder;
import org.quartz.JobBuilder;
import org.quartz.JobDetail;
import org.quartz.JobExecutionContext;
import org.quartz.JobKey;
import org.quartz.Scheduler;
import org.quartz.SchedulerException;
import org.quartz.Trigger;
import org.quartz.TriggerBuilder;
import org.quartz.TriggerKey;
import org.quartz.impl.StdSchedulerFactory;
import org.quartz.impl.matchers.GroupMatcher;

import org.slf4j.LoggerFactory;

public class QuartzSchedulerSingleton {
    private static QuartzSchedulerSingleton _instance = null;
    private static StdSchedulerFactory _schedulerFactory = null;
    private static Scheduler _scheduler = null;
    
    protected QuartzSchedulerSingleton() {
            System.out.println("QuartzSchedulerSingleton.constructor Start ");
            try {
                _schedulerFactory = new StdSchedulerFactory();
                String envFileLoc =System.getProperty("java.scratch.dir")+"/quartz.properties";
                _schedulerFactory.initialize(envFileLoc);

                _scheduler = _schedulerFactory.getScheduler();
                _scheduler.start();
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
            System.out.println("QuartzSchedulerSingleton.constructor End ");
        }
    
    public static QuartzSchedulerSingleton getInstance() {
            if (_instance == null) {
                System.out.println("QuartzSchedulerSingleton.getInstance create new ");
                _instance = new QuartzSchedulerSingleton();
            }
        org.slf4j.Logger logger;
        try {
            logger = LoggerFactory.getLogger(Class.forName("org.quartz.core.QuartzSchedulerThread"));
            if (logger != null && logger instanceof ch.qos.logback.classic.Logger) {                
                ch.qos.logback.classic.Logger log = (ch.qos.logback.classic.Logger)logger;
                System.out.println("Log level of Quartz..." + log.getLevel());
                
            }
        } catch (ClassNotFoundException e) {
        }
        
            return _instance;
    }
    public Scheduler getScheduler() {
          return _scheduler;
    }
    
    public boolean unscheduleJob(TriggerKey key,JobKey jobkey)throws RuntimeException {
        boolean deleteFlag = false;
        try {
            System.out.println("***unschedulJob and delete all triggers******");
            //System.out.println("Is Job unscheduled? " +_scheduler.unscheduleJob(key));//removing only the triggers based on trigger key
            System.out.println("No. of Jobs before delete...." +getScheduledJobs().size());
            deleteFlag = _scheduler.deleteJob(jobkey);
            System.out.println("Is Job deleted: " + deleteFlag); //deleting the job based on job key 
            System.out.println("No. of Jobs after delete...." +getScheduledJobs().size());

        } catch (SchedulerException e) {
            e.printStackTrace();
            deleteFlag = false;
            throw new RuntimeException(e);
        }
        return deleteFlag;
    }
    
    public List<ScheduledJobDTO> getScheduledJobs() {
        System.out.println("QuartzSchedulerSingleton.getScheduledJobs Start ");
        List<ScheduledJobDTO> result = new ArrayList<ScheduledJobDTO>();
        Date nextFireTime = null;
        TriggerKey tkey = null;
        try {            
            
            for (String groupName : _scheduler.getJobGroupNames()) {
                for (JobKey jobKey :_scheduler.getJobKeys(GroupMatcher.jobGroupEquals(groupName))) {
                     String jobName = jobKey.getName();
                     String jobGroup = jobKey.getGroup();

                   List<Trigger> triggers =  (List<Trigger>)_scheduler.getTriggersOfJob(jobKey);
                    //check list size to get triggers
                    
                    if(triggers.size() > 0){                        
                        nextFireTime = triggers.get(0).getNextFireTime();
                        tkey = triggers.get(0).getKey();                       
                    }  
                    else{
                        nextFireTime = null;
                        tkey = null;
                    }
                    JobDetail jobDetail = _scheduler.getJobDetail(jobKey); 
                    ScheduledJobDTO temp =  new ScheduledJobDTO(jobName, jobGroup, nextFireTime,jobDetail.getJobClass().getName(),_scheduler.getSchedulerName(),jobKey,tkey);
                   // System.out.println("QuartzSchedulerSingleton.getScheduledJobs job : "+temp);
                    result.add(temp);
                }
            }
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
        System.out.println("QuartzSchedulerSingleton.getScheduledJobs result "+result.size());
        return result;
    }
    
    /**
        * Schedule a job
        * @param jobName name of the job to be scheduled
        * @param jobClassName class of the job to be schedule
        * @param cronSchedule the cron based schedule for the job
        */
       public void scheduleJob(String jobName, String jobClassName, String groupName, String cronSchedule)throws RuntimeException {
           System.out.println("QuartzSchedulerSingleton.scheduleJob Start; "+jobName + " " + jobClassName + " " + cronSchedule);
           try {
               Class jobClass = Class.forName(jobClassName);

               // Create the job
               JobDetail job =
                   JobBuilder.newJob(jobClass).withIdentity(jobName,groupName).build();
               
               
               // Create a new cron based schedule 
               Trigger trigger =
                   TriggerBuilder.newTrigger().withIdentity(jobName,groupName).withSchedule(CronScheduleBuilder.cronSchedule(cronSchedule).withMisfireHandlingInstructionDoNothing()).build();//if there is a misfire then it will abort current job execution and schedule for next execution time
               
               _scheduler.scheduleJob(job,trigger);
           } catch (Exception e) {
               e.printStackTrace();
               throw new RuntimeException(e);
           }
           System.out.println("QuartzSchedulerSingleton.scheduleJob End ");
       }
    
    /**
     * Trigger a job immediately without scheduling it
     */
    public void triggerJobImmediately(String jobName,String groupName,String jobClassName)throws RuntimeException {
        System.out.println("*************triggerJobImmediately*************");
        Class jobClass;
        try {
            jobClass = Class.forName(jobClassName);
            JobDetail job = JobBuilder.newJob(jobClass).withIdentity(jobName,groupName).storeDurably(true).build();
            
            _scheduler.addJob(job, true);//register the job in scheduler
            _scheduler.triggerJob(job.getKey());//trigger the job immediately
            System.out.println("Currently executing jobs... " +_scheduler.getCurrentlyExecutingJobs().size());
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }        
    }
    /**
     * Cancel a job which is running
     */
    public void cancelJob(JobKey jobkey)throws RuntimeException {
        System.out.println("Trying to cancel a job...."+jobkey.getName()+" "+jobkey.getGroup());
        try{
            _scheduler.interrupt(jobkey);
        } catch (Exception e) {
            e.printStackTrace();
            throw new RuntimeException(e);
        }  
    }
    public List<ScheduledJobDTO> getCurrentlyExecutingJobs()throws SchedulerException{
       java.util.List<org.quartz.JobExecutionContext> executingJobs = _scheduler.getCurrentlyExecutingJobs();
       List<ScheduledJobDTO> result = new ArrayList<ScheduledJobDTO>();
       System.out.println("No. of currently executing jobs..."+result.size());
       for(JobExecutionContext jobCtx : executingJobs){
           JobDetail jobDtl = jobCtx.getJobDetail();           
           ScheduledJobDTO temp =  new ScheduledJobDTO(jobDtl.getKey().getName(), jobDtl.getKey().getGroup(), jobCtx.getNextFireTime(),jobDtl.getJobClass().getName(),_scheduler.getSchedulerName(),jobDtl.getKey(),jobCtx.getTrigger().getKey());
           result.add(temp);
       }
       return result;
    }
}
